@@ -21,7 +21,8 @@ from neutron_lib.exceptions import l3 as l3_exc
 from neutron_lib.utils import helpers
 from oslo_log import log as logging
 from oslo_utils import netutils
-from pyroute2.netlink import exceptions as pyroute2_exc
+from pyroute2.netlink import exceptions \
+    as pyroute2_exc  # pylint: disable=no-name-in-module
 
 from neutron._i18n import _
 from neutron.agent.l3 import namespaces
@@ -186,20 +187,71 @@ class RouterInfo(BaseRouterInfo):
     def update_routing_table(self, operation, route):
         self._update_routing_table(operation, route, self.ns_name)
 
+    def update_routing_table_ecmp(self, route_list):
+        multipath = [dict(via=route['nexthop'])
+                     for route in route_list]
+        try:
+            ip_lib.add_ip_route(self.ns_name, route_list[0]['destination'],
+                                via=multipath)
+        except (RuntimeError, OSError, pyroute2_exc.NetlinkError):
+            pass
+
+    def check_and_remove_ecmp_route(self, old_routes, remove_route):
+        route_list = []
+        for route in old_routes:
+            if route['destination'] == remove_route['destination']:
+                route_list.append(route)
+        # An ECMP route is composed of multiple routes with the same
+        # destination address, and two scenarios should be considered
+        # when removing a nexthop address from an ECMP route.
+        # a. The original ECMP route has only two nexthops, deleting
+        #    one of them will make it a normal route.
+        # b. The original ECMP route has more than two nexthops,
+        #    delete one of the nexthops, it is still an ECMP route.
+        if len(route_list) == 2:
+            for r in route_list:
+                if r['nexthop'] != remove_route['nexthop']:
+                    self.update_routing_table('replace', r)
+            return True
+
+        if len(route_list) > 2:
+            route_list.remove(remove_route)
+            self.update_routing_table_ecmp(route_list)
+            return True
+
+        return False
+
+    def check_and_add_ecmp_route(self, old_routes, new_route):
+        route_list = []
+        for route in old_routes:
+            if route['destination'] == new_route['destination']:
+                route_list.append(route)
+
+        if route_list:
+            route_list.append(new_route)
+            self.update_routing_table_ecmp(route_list)
+            return True
+
+        return False
+
     def routes_updated(self, old_routes, new_routes):
         adds, removes = helpers.diff_list_of_dict(old_routes,
                                                   new_routes)
-        for route in adds:
-            LOG.debug("Added route entry is '%s'", route)
-            # remove replaced route from deleted route
-            for del_route in removes:
-                if route['destination'] == del_route['destination']:
-                    removes.remove(del_route)
-            # replace success even if there is no existing route
-            self.update_routing_table('replace', route)
         for route in removes:
-            LOG.debug("Removed route entry is '%s'", route)
-            self.update_routing_table('delete', route)
+            # Judge if modifying an ECMP route or not, if not,
+            # just delete it, if it is, replace it
+            # update old_routes after modify
+            if not self.check_and_remove_ecmp_route(old_routes, route):
+                LOG.debug("Removed route entry is '%s'", route)
+                self.update_routing_table('delete', route)
+            old_routes.remove(route)
+
+        for route in adds:
+            if not self.check_and_add_ecmp_route(old_routes, route):
+                LOG.debug("Added route entry is '%s'", route)
+                # replace success even if there is no existing route
+                self.update_routing_table('replace', route)
+            old_routes.append(route)
 
     def get_floating_ips(self):
         """Filter Floating IPs to be hosted on this agent."""
@@ -589,7 +641,10 @@ class RouterInfo(BaseRouterInfo):
                       self.router_id)
             self.radvd.disable()
 
-    def internal_network_updated(self, interface_name, ip_cidrs, mtu):
+    def internal_network_updated(self, port):
+        interface_name = self.get_internal_device_name(port['id'])
+        ip_cidrs = common_utils.fixed_ip_cidrs(port['fixed_ips'])
+        mtu = port['mtu']
         self.driver.set_mtu(interface_name, mtu, namespace=self.ns_name,
                             prefix=INTERNAL_DEV_PREFIX)
         self.driver.init_router_port(
@@ -648,12 +703,8 @@ class RouterInfo(BaseRouterInfo):
         updated_cidrs = []
         for p in updated_ports:
             self._update_internal_ports_cache(p)
-            interface_name = self.get_internal_device_name(p['id'])
-            ip_cidrs = common_utils.fixed_ip_cidrs(p['fixed_ips'])
-            LOG.debug("updating internal network for port %s", p)
-            updated_cidrs += ip_cidrs
-            self.internal_network_updated(
-                interface_name, ip_cidrs, p['mtu'])
+            updated_cidrs += common_utils.fixed_ip_cidrs(p['fixed_ips'])
+            self.internal_network_updated(p)
             enable_ra = enable_ra or self._port_has_ipv6_subnet(p)
 
         # Check if there is any pd prefix update
@@ -699,16 +750,14 @@ class RouterInfo(BaseRouterInfo):
         return [common_utils.ip_to_cidr(ip['floating_ip_address'])
                 for ip in floating_ips]
 
-    def _plug_external_gateway(self, ex_gw_port, interface_name, ns_name,
-                               link_up=True):
+    def _plug_external_gateway(self, ex_gw_port, interface_name, ns_name):
         self.driver.plug(ex_gw_port['network_id'],
                          ex_gw_port['id'],
                          interface_name,
                          ex_gw_port['mac_address'],
                          namespace=ns_name,
                          prefix=EXTERNAL_DEV_PREFIX,
-                         mtu=ex_gw_port.get('mtu'),
-                         link_up=link_up)
+                         mtu=ex_gw_port.get('mtu'))
 
     def _get_external_gw_ips(self, ex_gw_port):
         gateway_ips = []
@@ -768,11 +817,7 @@ class RouterInfo(BaseRouterInfo):
         LOG.debug("External gateway added: port(%s), interface(%s), ns(%s)",
                   ex_gw_port, interface_name, ns_name)
         self._plug_external_gateway(ex_gw_port, interface_name, ns_name)
-        self._external_gateway_settings(ex_gw_port, interface_name,
-                                        ns_name, preserve_ips)
 
-    def _external_gateway_settings(self, ex_gw_port, interface_name,
-                                   ns_name, preserve_ips):
         # Build up the interface and gateway IP addresses that
         # will be added to the interface.
         ip_cidrs = common_utils.fixed_ip_cidrs(ex_gw_port['fixed_ips'])
@@ -817,19 +862,17 @@ class RouterInfo(BaseRouterInfo):
         return any(netaddr.IPAddress(gw_ip).version == 6
                    for gw_ip in gateway_ips)
 
-    def get_router_preserve_ips(self):
+    def external_gateway_added(self, ex_gw_port, interface_name):
         preserve_ips = self._list_floating_ip_cidrs() + list(
             self.centralized_port_forwarding_fip_set)
         preserve_ips.extend(self.agent.pd.get_preserve_ips(self.router_id))
-        return preserve_ips
-
-    def external_gateway_added(self, ex_gw_port, interface_name):
-        preserve_ips = self.get_router_preserve_ips()
         self._external_gateway_added(
             ex_gw_port, interface_name, self.ns_name, preserve_ips)
 
     def external_gateway_updated(self, ex_gw_port, interface_name):
-        preserve_ips = self.get_router_preserve_ips()
+        preserve_ips = self._list_floating_ip_cidrs() + list(
+            self.centralized_port_forwarding_fip_set)
+        preserve_ips.extend(self.agent.pd.get_preserve_ips(self.router_id))
         self._external_gateway_added(
             ex_gw_port, interface_name, self.ns_name, preserve_ips)
 
@@ -1089,8 +1132,14 @@ class RouterInfo(BaseRouterInfo):
              'interface_name': INTERNAL_DEV_PREFIX + '+',
              'value': self.agent_conf.metadata_access_mark,
              'mask': lib_constants.ROUTER_MARK_MASK})
+        drop_non_local_metadata = (
+            '-m mark --mark %s/%s -j DROP' % (
+                self.agent_conf.metadata_access_mark,
+                lib_constants.ROUTER_MARK_MASK))
         self.iptables_manager.ipv4['mangle'].add_rule(
             'PREROUTING', mark_metadata_for_internal_interfaces)
+        self.iptables_manager.ipv4['filter'].add_rule(
+            'scope', drop_non_local_metadata)
 
         if netutils.is_ipv6_enabled():
             mark_metadata_v6_for_internal_interfaces = (
@@ -1102,14 +1151,24 @@ class RouterInfo(BaseRouterInfo):
                  'interface_name': INTERNAL_DEV_PREFIX + '+',
                  'value': self.agent_conf.metadata_access_mark,
                  'mask': lib_constants.ROUTER_MARK_MASK})
+            drop_non_local_v6_metadata = (
+                '-m mark --mark %s/%s -j DROP' % (
+                    self.agent_conf.metadata_access_mark,
+                    lib_constants.ROUTER_MARK_MASK))
             self.iptables_manager.ipv6['mangle'].add_rule(
                 'PREROUTING', mark_metadata_v6_for_internal_interfaces)
+            self.iptables_manager.ipv6['filter'].add_rule(
+                'scope', drop_non_local_v6_metadata)
 
-    def _get_port_devicename_scopemark(self, ports, name_generator):
+    def _get_port_devicename_scopemark(
+            self, ports, name_generator, interface_name=None):
         devicename_scopemark = {lib_constants.IP_VERSION_4: dict(),
                                 lib_constants.IP_VERSION_6: dict()}
         for p in ports:
-            device_name = name_generator(p['id'])
+            if interface_name is None:
+                device_name = name_generator(p['id'])
+            else:
+                device_name = interface_name
             ip_cidrs = common_utils.fixed_ip_cidrs(p['fixed_ips'])
             port_as_marks = self.get_port_address_scope_mark(p)
             for ip_version in {common_utils.get_ip_version(cidr)
