@@ -258,6 +258,12 @@ class SecurityGroupServerAPIShim(sg_rpc_base.SecurityGroupInfoAPIMixin):
                            'Port', events.AFTER_DELETE)
         registry.subscribe(self._handle_sg_member_update,
                            'Port', events.AFTER_UPDATE)
+        self._register_legacy_ag_notification_callbacks(sg_agent)
+
+    def _register_legacy_ag_notification_callbacks(self, sg_agent):
+        for event in (events.AFTER_UPDATE, events.AFTER_DELETE):
+            registry.subscribe(self._handle_address_group_event,
+                               resources.ADDRESSGROUP, event)
 
     def security_group_info_for_devices(self, context, devices):
         ports = self._get_devices_info(context, devices)
@@ -270,17 +276,24 @@ class SecurityGroupServerAPIShim(sg_rpc_base.SecurityGroupInfoAPIMixin):
         # error.
         raise NotImplementedError()
 
-    def _add_child_sg_rules(self, rtype, event, trigger, context, updated,
-                            **kwargs):
+    def get_secgroup_ids_for_address_group(self, address_group_id):
+        filters = {'remote_address_group_id': (address_group_id, )}
+        return set([rule.security_group_id for rule in
+                    self.rcache.get_resources('SecurityGroupRule', filters)])
+
+    def _add_child_sg_rules(self, rtype, event, trigger, payload):
         # whenever we receive a full security group, add all child rules
         # because the server won't emit events for the individual rules on
         # creation.
+        context = payload.context
+        updated = payload.latest_state
         for rule in updated.rules:
             self.rcache.record_resource_update(context, 'SecurityGroupRule',
                                                rule)
 
-    def _clear_child_sg_rules(self, rtype, event, trigger, context, existing,
-                              **kwargs):
+    def _clear_child_sg_rules(self, rtype, event, trigger, payload):
+        context = payload.context
+        existing = payload.states[0]
         if not existing:
             return
         # the server can delete an entire security group without notifying
@@ -290,29 +303,38 @@ class SecurityGroupServerAPIShim(sg_rpc_base.SecurityGroupInfoAPIMixin):
         for rule in self.rcache.get_resources('SecurityGroupRule', filters):
             self.rcache.record_resource_delete(context, 'SecurityGroupRule',
                                                rule.id)
+        # If there's a rule which remote is the deleted sg, remove that also.
+        rules = self.rcache.match_resources_with_func(
+            'SecurityGroupRule',
+            lambda sg_rule: sg_rule.remote_group_id == existing.id)
+        for rule in rules:
+            self.rcache.record_resource_delete(context, 'SecurityGroupRule',
+                                               rule.id)
 
-    def _handle_sg_rule_delete(self, rtype, event, trigger, context, existing,
-                               **kwargs):
+    def _handle_sg_rule_delete(self, rtype, event, trigger, payload):
+        existing = payload.states[0]
         if not existing:
             return
         sg_id = existing.security_group_id
         self._sg_agent.security_groups_rule_updated([sg_id])
 
-    def _handle_sg_rule_update(self, rtype, event, trigger, context, existing,
-                               updated, **kwargs):
+    def _handle_sg_rule_update(self, rtype, event, trigger, payload):
+        updated = payload.latest_state
         sg_id = updated.security_group_id
         self._sg_agent.security_groups_rule_updated([sg_id])
 
-    def _handle_sg_member_delete(self, rtype, event, trigger, context,
-                                 existing, **kwargs):
+    def _handle_sg_member_delete(self, rtype, event, trigger, payload):
         # received on port delete
+        existing = payload.states[0]
         sgs = set(existing.security_group_ids) if existing else set()
         if sgs:
             self._sg_agent.security_groups_member_updated(sgs)
 
-    def _handle_sg_member_update(self, rtype, event, trigger, context,
-                                 existing, updated, changed_fields, **kwargs):
+    def _handle_sg_member_update(self, rtype, event, trigger, payload):
         # received on port update
+        existing = payload.states[0]
+        updated = payload.latest_state
+        changed_fields = payload.metadata['changed_fields']
         sgs = set(existing.security_group_ids) if existing else set()
         if not changed_fields.intersection({'security_group_ids', 'fixed_ips',
                                             'allowed_address_pairs'}):
@@ -322,6 +344,13 @@ class SecurityGroupServerAPIShim(sg_rpc_base.SecurityGroupInfoAPIMixin):
         if sgs:
             self._sg_agent.security_groups_member_updated(sgs)
 
+    def _handle_address_group_event(self, rtype, event, trigger, payload):
+        resource_id = payload.resource_id
+        if event == events.AFTER_UPDATE:
+            self._sg_agent.address_group_updated(resource_id)
+        else:
+            self._sg_agent.address_group_deleted(resource_id)
+
     def _get_devices_info(self, context, devices):
         # NOTE(kevinbenton): this format is required by the sg code, it is
         # defined in get_port_from_device and mimics
@@ -339,6 +368,7 @@ class SecurityGroupServerAPIShim(sg_rpc_base.SecurityGroupInfoAPIMixin):
             port['security_groups'] = list(ovo.security_group_ids)
             port['security_group_rules'] = []
             port['security_group_source_groups'] = []
+            port['security_group_remote_address_groups'] = []
             port['fixed_ips'] = [str(f['ip_address'])
                                  for f in port['fixed_ips']]
             # NOTE(kevinbenton): this id==device is only safe for OVS. a lookup
@@ -366,6 +396,19 @@ class SecurityGroupServerAPIShim(sg_rpc_base.SecurityGroupInfoAPIMixin):
                     ips_by_group[sg_id].update(set(port_ips))
         return ips_by_group
 
+    def _select_ips_for_remote_address_group(self, context,
+                                             remote_address_group_ids):
+        if not remote_address_group_ids:
+            return {}
+        ips_by_group = {ag: set() for ag in remote_address_group_ids}
+        for ag_id in remote_address_group_ids:
+            ag = self.rcache.get_resource_by_id('AddressGroup', ag_id)
+            # NOTE(hangyang) Expected data structure is (ip, mac) tuple
+            ips = [(str(addr_assoc.address), None)
+                   for addr_assoc in ag.addresses]
+            ips_by_group[ag_id].update(set(ips))
+        return ips_by_group
+
     def _select_rules_for_ports(self, context, ports):
         if not ports:
             return []
@@ -388,6 +431,10 @@ class SecurityGroupServerAPIShim(sg_rpc_base.SecurityGroupInfoAPIMixin):
                       for sg_id in p['security_group_ids']))
         return [(sg_id, ) for sg_id in sg_ids]
 
-    def _is_security_group_stateful(self, context, sg_id):
-        sg = self.rcache.get_resource_by_id(resources.SECURITYGROUP, sg_id)
-        return sg.stateful
+    def _get_sgs_stateful_flag(self, context, sg_ids):
+        sgs_stateful = {}
+        for sg_id in sg_ids:
+            sg = self.rcache.get_resource_by_id(resources.SECURITYGROUP, sg_id)
+            sgs_stateful[sg_id] = sg.stateful
+
+        return sgs_stateful
