@@ -14,17 +14,28 @@ from neutron_lib.db import model_base
 import sqlalchemy as sa
 from sqlalchemy import orm
 
-from neutron.db import agents_db
+from neutron.common import _constants as n_const
+from neutron.db.models import agent as agent_model
 
 
 class NetworkDhcpAgentBinding(model_base.BASEV2):
     """Represents binding between neutron networks and DHCP agents."""
 
+    __table_args__ = (
+        sa.UniqueConstraint(
+            'network_id', 'binding_index',
+            name='uniq_network_dhcp_agent_binding0network_id0binding_index0'),
+        model_base.BASEV2.__table_args__
+    )
+
     network_id = sa.Column(sa.String(36),
                            sa.ForeignKey("networks.id", ondelete='CASCADE'),
                            primary_key=True)
-    dhcp_agent = orm.relation(agents_db.Agent)
+    dhcp_agent = orm.relation(agent_model.Agent, lazy='subquery')
     dhcp_agent_id = sa.Column(sa.String(36),
                               sa.ForeignKey("agents.id",
                                             ondelete='CASCADE'),
                               primary_key=True)
+    binding_index = sa.Column(
+        sa.Integer, nullable=False,
+        server_default=str(n_const.LOWEST_AGENT_BINDING_INDEX))
