@@ -89,6 +89,16 @@ def check_asserttruefalse(logical_line, filename):
             yield (0, msg)
 
 
+@core.flake8ext
+def check_assertitemsequal(logical_line, filename):
+    """N329 - Don't use assertItemsEqual."""
+    if 'neutron/tests/' in filename:
+        if re.search(r"assertItemsEqual[\(,]", logical_line):
+            msg = ("N329: Use assertCountEqual() instead of "
+                   "assertItemsEqual()")
+            yield (0, msg)
+
+
 @core.flake8ext
 def check_assertempty(logical_line, filename):
     """N330 - Enforce using assertEqual parameter ordering in case of empty
@@ -117,7 +127,7 @@ def check_assertisinstance(logical_line, filename):
 
 @core.flake8ext
 def check_assertequal_for_httpcode(logical_line, filename):
-    """N332 - Enforce correct oredering for httpcode in assertEqual."""
+    """N332 - Enforce correct ordering for httpcode in assertEqual."""
     msg = ("N332: Use assertEqual(expected_http_code, observed_http_code) "
            "instead of assertEqual(observed_http_code, expected_http_code)")
     if 'neutron/tests/' in filename:
