@@ -18,12 +18,12 @@ import operator
 
 import netaddr
 from neutron_lib import constants
-from neutron_lib import exceptions as lib_exc
+from neutron_lib.db import api as db_api
+from neutron_lib import exceptions
 from oslo_db import exception as db_exc
 from oslo_utils import uuidutils
 
 from neutron._i18n import _
-from neutron.common import exceptions as n_exc
 from neutron.db import models_v2
 from neutron.ipam import driver
 from neutron.ipam import exceptions as ipam_exc
@@ -49,29 +49,34 @@ class SubnetAllocator(driver.Pool):
         subnetpool, it's required to ensure non-overlapping cidrs in the same
         subnetpool.
         """
-
-        current_hash = (self._context.session.query(models_v2.SubnetPool.hash)
-                        .filter_by(id=self._subnetpool['id']).scalar())
+        with db_api.CONTEXT_READER.using(self._context):
+            current_hash = (
+                self._context.session.query(models_v2.SubnetPool.hash)
+                .filter_by(id=self._subnetpool['id']).scalar())
         if current_hash is None:
             # NOTE(cbrandily): subnetpool has been deleted
-            raise n_exc.SubnetPoolNotFound(
+            raise exceptions.SubnetPoolNotFound(
                 subnetpool_id=self._subnetpool['id'])
         new_hash = uuidutils.generate_uuid()
 
         # NOTE(cbrandily): the update disallows 2 concurrent subnet allocation
         # to succeed: at most 1 transaction will succeed, others will be
         # rolled back and be caught in neutron.db.v2.base
-        query = self._context.session.query(models_v2.SubnetPool).filter_by(
-            id=self._subnetpool['id'], hash=current_hash)
-        count = query.update({'hash': new_hash})
+        with db_api.CONTEXT_WRITER.using(self._context):
+            query = (
+                self._context.session.query(models_v2.SubnetPool).filter_by(
+                    id=self._subnetpool['id'], hash=current_hash))
+
+            count = query.update({'hash': new_hash})
         if not count:
-            raise db_exc.RetryRequest(lib_exc.SubnetPoolInUse(
+            raise db_exc.RetryRequest(exceptions.SubnetPoolInUse(
                                       subnet_pool_id=self._subnetpool['id']))
 
     def _get_allocated_cidrs(self):
-        query = self._context.session.query(models_v2.Subnet)
-        subnets = query.filter_by(subnetpool_id=self._subnetpool['id'])
-        return (x.cidr for x in subnets)
+        with db_api.CONTEXT_READER.using(self._context):
+            query = self._context.session.query(models_v2.Subnet.cidr)
+            subnets = query.filter_by(subnetpool_id=self._subnetpool['id'])
+            return (x.cidr for x in subnets)
 
     def _get_available_prefix_list(self):
         prefixes = (x.cidr for x in self._subnetpool.prefixes)
@@ -90,8 +95,8 @@ class SubnetAllocator(driver.Pool):
     def _allocations_used_by_tenant(self, quota_unit):
         subnetpool_id = self._subnetpool['id']
         tenant_id = self._subnetpool['tenant_id']
-        with self._context.session.begin(subtransactions=True):
-            qry = self._context.session.query(models_v2.Subnet)
+        with db_api.CONTEXT_READER.using(self._context):
+            qry = self._context.session.query(models_v2.Subnet.cidr)
             allocations = qry.filter_by(subnetpool_id=subnetpool_id,
                                         tenant_id=tenant_id)
             value = 0
@@ -112,10 +117,10 @@ class SubnetAllocator(driver.Pool):
                                                                  quota_unit)
 
             if used + requested_units > quota:
-                raise n_exc.SubnetPoolQuotaExceeded()
+                raise exceptions.SubnetPoolQuotaExceeded()
 
     def _allocate_any_subnet(self, request):
-        with self._context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(self._context):
             self._lock_subnetpool()
             self._check_subnetpool_tenant_quota(request.tenant_id,
                                                 request.prefixlen)
@@ -135,18 +140,18 @@ class SubnetAllocator(driver.Pool):
                                       gateway_ip=gateway_ip,
                                       allocation_pools=pools)
             msg = _("Insufficient prefix space to allocate subnet size /%s")
-            raise n_exc.SubnetAllocationError(reason=msg %
-                                              str(request.prefixlen))
+            raise exceptions.SubnetAllocationError(
+                reason=msg % str(request.prefixlen))
 
     def _allocate_specific_subnet(self, request):
-        with self._context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(self._context):
             self._lock_subnetpool()
             self._check_subnetpool_tenant_quota(request.tenant_id,
                                                 request.prefixlen)
             cidr = request.subnet_cidr
             available = self._get_available_prefix_list()
             matched = netaddr.all_matching_cidrs(cidr, available)
-            if len(matched) is 1 and matched[0].prefixlen <= cidr.prefixlen:
+            if len(matched) == 1 and matched[0].prefixlen <= cidr.prefixlen:
                 return IpamSubnet(request.tenant_id,
                                   request.subnet_id,
                                   cidr,
@@ -154,17 +159,17 @@ class SubnetAllocator(driver.Pool):
                                   allocation_pools=request.allocation_pools)
             msg = _("Cannot allocate requested subnet from the available "
                     "set of prefixes")
-            raise n_exc.SubnetAllocationError(reason=msg)
+            raise exceptions.SubnetAllocationError(reason=msg)
 
     def allocate_subnet(self, request):
         max_prefixlen = int(self._subnetpool['max_prefixlen'])
         min_prefixlen = int(self._subnetpool['min_prefixlen'])
         if request.prefixlen > max_prefixlen:
-            raise n_exc.MaxPrefixSubnetAllocationError(
+            raise exceptions.MaxPrefixSubnetAllocationError(
                               prefixlen=request.prefixlen,
                               max_prefixlen=max_prefixlen)
         if request.prefixlen < min_prefixlen:
-            raise n_exc.MinPrefixSubnetAllocationError(
+            raise exceptions.MinPrefixSubnetAllocationError(
                               prefixlen=request.prefixlen,
                               min_prefixlen=min_prefixlen)
 
@@ -174,7 +179,7 @@ class SubnetAllocator(driver.Pool):
             return self._allocate_specific_subnet(request)
         else:
             msg = _("Unsupported request type")
-            raise n_exc.SubnetAllocationError(reason=msg)
+            raise exceptions.SubnetAllocationError(reason=msg)
 
     def get_subnet(self, subnet_id):
         raise NotImplementedError()
@@ -224,7 +229,7 @@ class IpamSubnetGroup(driver.SubnetGroup):
            to try to allocate an IP from each subnet in turn, one by one.  This
            implementation preserves that behavior so that existing drivers work
            as they did before while giving them the opportunity to optimize it
-           by overridding the implementation.
+           by overriding the implementation.
         '''
         for subnet_id in self._subnet_ids:
             try:
@@ -258,7 +263,7 @@ class SubnetPoolReader(object):
         self._read_address_scope(subnetpool)
         self.subnetpool = {'id': self.id,
                            'name': self.name,
-                           'tenant_id': self.tenant_id,
+                           'project_id': self.tenant_id,
                            'prefixes': self.prefixes,
                            'min_prefix': self.min_prefix,
                            'min_prefixlen': self.min_prefixlen,
@@ -308,8 +313,8 @@ class SubnetPoolReader(object):
         self._sp_helper.validate_max_prefixlen(self.max_prefixlen,
                                                ip_version)
         self._sp_helper.validate_default_prefixlen(self.min_prefixlen,
-                                                self.max_prefixlen,
-                                                self.default_prefixlen)
+                                                   self.max_prefixlen,
+                                                   self.default_prefixlen)
 
     def _read_prefix_bound(self, type, subnetpool, default_bound=None):
         prefixlen_attr = type + '_prefixlen'
@@ -330,14 +335,14 @@ class SubnetPoolReader(object):
     def _read_prefix_info(self, subnetpool):
         prefix_list = subnetpool['prefixes']
         if not prefix_list:
-            raise n_exc.EmptySubnetPoolPrefixList()
+            raise exceptions.EmptySubnetPoolPrefixList()
 
         ip_version = None
         for prefix in prefix_list:
             if not ip_version:
                 ip_version = netaddr.IPNetwork(prefix).version
             elif netaddr.IPNetwork(prefix).version != ip_version:
-                raise n_exc.PrefixVersionMismatch()
+                raise exceptions.PrefixVersionMismatch()
         self.default_quota = subnetpool.get('default_quota')
 
         if self.default_quota is constants.ATTR_NOT_SPECIFIED:
@@ -347,8 +352,11 @@ class SubnetPoolReader(object):
         self.prefixes = self._compact_subnetpool_prefix_list(prefix_list)
 
     def _read_address_scope(self, subnetpool):
-        self.address_scope_id = subnetpool.get('address_scope_id',
-                                               constants.ATTR_NOT_SPECIFIED)
+        address_scope_id = subnetpool.get('address_scope_id',
+                                          constants.ATTR_NOT_SPECIFIED)
+        if address_scope_id is constants.ATTR_NOT_SPECIFIED:
+            address_scope_id = None
+        self.address_scope_id = address_scope_id
 
     def _compact_subnetpool_prefix_list(self, prefix_list):
         """Compact any overlapping prefixes in prefix_list and return the
@@ -364,22 +372,22 @@ class SubnetPoolReader(object):
 class SubnetPoolHelper(object):
 
     _PREFIX_VERSION_INFO = {4: {'max_prefixlen': constants.IPv4_BITS,
-                               'wildcard': '0.0.0.0',
-                               'default_min_prefixlen': 8,
-                               # IPv4 quota measured in units of /32
-                               'quota_units': 32},
-                           6: {'max_prefixlen': constants.IPv6_BITS,
-                               'wildcard': '::',
-                               'default_min_prefixlen': 64,
-                               # IPv6 quota measured in units of /64
-                               'quota_units': 64}}
+                                'wildcard': '0.0.0.0',
+                                'default_min_prefixlen': 8,
+                                # IPv4 quota measured in units of /32
+                                'quota_units': 32},
+                            6: {'max_prefixlen': constants.IPv6_BITS,
+                                'wildcard': '::',
+                                'default_min_prefixlen': 64,
+                                # IPv6 quota measured in units of /64
+                                'quota_units': 64}}
 
     def validate_min_prefixlen(self, min_prefixlen, max_prefixlen):
         if min_prefixlen < 0:
-            raise n_exc.UnsupportedMinSubnetPoolPrefix(prefix=min_prefixlen,
-                                                       version=4)
+            raise exceptions.UnsupportedMinSubnetPoolPrefix(
+                prefix=min_prefixlen, version=4)
         if min_prefixlen > max_prefixlen:
-            raise n_exc.IllegalSubnetPoolPrefixBounds(
+            raise exceptions.IllegalSubnetPoolPrefixBounds(
                                              prefix_type='min_prefixlen',
                                              prefixlen=min_prefixlen,
                                              base_prefix_type='max_prefixlen',
@@ -388,7 +396,7 @@ class SubnetPoolHelper(object):
     def validate_max_prefixlen(self, prefixlen, ip_version):
         max = self._PREFIX_VERSION_INFO[ip_version]['max_prefixlen']
         if prefixlen > max:
-            raise n_exc.IllegalSubnetPoolPrefixBounds(
+            raise exceptions.IllegalSubnetPoolPrefixBounds(
                                             prefix_type='max_prefixlen',
                                             prefixlen=prefixlen,
                                             base_prefix_type='ip_version_max',
@@ -399,13 +407,13 @@ class SubnetPoolHelper(object):
                                    max_prefixlen,
                                    default_prefixlen):
         if default_prefixlen < min_prefixlen:
-            raise n_exc.IllegalSubnetPoolPrefixBounds(
+            raise exceptions.IllegalSubnetPoolPrefixBounds(
                                              prefix_type='default_prefixlen',
                                              prefixlen=default_prefixlen,
                                              base_prefix_type='min_prefixlen',
                                              base_prefixlen=min_prefixlen)
         if default_prefixlen > max_prefixlen:
-            raise n_exc.IllegalSubnetPoolPrefixBounds(
+            raise exceptions.IllegalSubnetPoolPrefixBounds(
                                              prefix_type='default_prefixlen',
                                              prefixlen=default_prefixlen,
                                              base_prefix_type='max_prefixlen',
