@@ -120,6 +120,13 @@ def delete_distributed_port_binding_if_stale(context, binding):
         with db_api.CONTEXT_WRITER.using(context):
             LOG.debug("Distributed port: Deleting binding %s", binding)
             context.session.delete(binding)
+            for bindlv in (context.session.query(models.PortBindingLevel).
+                    filter_by(port_id=binding.port_id, host=binding.host)):
+                context.session.delete(bindlv)
+            LOG.debug("For port %(port_id)s, host %(host)s, "
+                      "cleared binding levels",
+                      {'port_id': binding.port_id,
+                       'host': binding.host})
 
 
 def get_port(context, port_id):
@@ -142,13 +149,6 @@ def get_port(context, port_id):
             return
 
 
-@db_api.CONTEXT_READER
-def get_port_from_device_mac(context, device_mac):
-    LOG.debug("get_port_from_device_mac() called for mac %s", device_mac)
-    ports = port_obj.Port.get_objects(context, mac_address=device_mac)
-    return ports.pop() if ports else None
-
-
 def get_ports_and_sgs(context, port_ids):
     """Get ports from database with security group info."""
 
@@ -206,6 +206,7 @@ def make_port_dict_with_security_groups(port, sec_groups):
     port_dict['security_groups'] = sec_groups
     port_dict['security_group_rules'] = []
     port_dict['security_group_source_groups'] = []
+    port_dict['security_group_remote_address_groups'] = []
     port_dict['fixed_ips'] = [ip['ip_address']
                               for ip in port['fixed_ips']]
     return port_dict
@@ -255,6 +256,17 @@ def get_distributed_port_binding_by_host(context, port_id, host):
     return binding
 
 
+def update_distributed_port_binding_by_host(context, port_id, host, router_id):
+    with db_api.CONTEXT_WRITER.using(context):
+        bindings = (
+            context.session.query(models.DistributedPortBinding).
+            filter(models.DistributedPortBinding.port_id.startswith(port_id),
+                   models.DistributedPortBinding.host == host).all())
+        for binding in bindings or []:
+            binding['router_id'] = router_id or None
+            binding.update(binding)
+
+
 def get_distributed_port_bindings(context, port_id):
     with db_api.CONTEXT_READER.using(context):
         bindings = (context.session.query(models.DistributedPortBinding).
@@ -321,10 +333,9 @@ def _prevent_segment_delete_with_port_bound(resource, event, trigger,
         # don't check for network deletes
         return
 
-    with db_api.CONTEXT_READER.using(payload.context):
-        auto_delete_port_ids, proper_port_count = port_obj.Port.\
-            get_auto_deletable_port_ids_and_proper_port_count_by_segment(
-                payload.context, segment_id=payload.resource_id)
+    auto_delete_port_ids, proper_port_count = port_obj.Port.\
+        get_auto_deletable_port_ids_and_proper_port_count_by_segment(
+            payload.context, segment_id=payload.resource_id)
 
     if proper_port_count:
         reason = (_("The segment is still bound with %s port(s)") %
