@@ -10,13 +10,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import contextlib
-
 from neutron_lib import constants
 from neutron_lib import exceptions
 from oslo_log import log as logging
 
-from neutron._i18n import _, _LE
+from neutron._i18n import _
 from neutron.agent.common import ovs_lib
 from neutron.services.trunk.drivers.openvswitch.agent import exceptions as exc
 from neutron.services.trunk.drivers.openvswitch import utils
@@ -51,7 +49,7 @@ def get_patch_peer_attrs(peer_name, port_mac=None, port_id=None):
     if port_id:
         external_ids['iface-id'] = port_id
     attrs = [('type', 'patch'),
-            ('options', {'peer': peer_name})]
+             ('options', {'peer': peer_name})]
     if external_ids:
         attrs.append(
             ('external_ids', external_ids))
@@ -93,26 +91,7 @@ class TrunkParentPort(object):
             self.DEV_PREFIX, port_id)
         self._transaction = None
 
-    # TODO(jlibosva): Move nested transaction to ovs_lib
-    @contextlib.contextmanager
-    def ovsdb_transaction(self):
-        """Context manager for ovsdb transaction.
-
-        The object caches whether its already in transaction and if it is, the
-        original transaction is returned.  This behavior enables calling
-        manager several times while always getting the same transaction.
-        """
-        if self._transaction:
-            yield self._transaction
-        else:
-            with self.bridge.ovsdb.transaction() as txn:
-                self._transaction = txn
-                try:
-                    yield txn
-                finally:
-                    self._transaction = None
-
-    def plug(self, br_int):
+    def plug(self, br_int, tag=0):
         """Plug patch ports between trunk bridge and given bridge.
 
         The method plugs one patch port on the given bridge side using
@@ -136,7 +115,7 @@ class TrunkParentPort(object):
         patch_trunk_attrs = get_patch_peer_attrs(self.patch_port_int_name,
                                                  self.port_mac, self.port_id)
 
-        with self.ovsdb_transaction() as txn:
+        with ovsdb.transaction() as txn:
             txn.add(ovsdb.add_port(br_int.br_name,
                                    self.patch_port_int_name))
             txn.add(ovsdb.db_set('Interface', self.patch_port_int_name,
@@ -145,6 +124,9 @@ class TrunkParentPort(object):
                                    self.patch_port_trunk_name))
             txn.add(ovsdb.db_set('Interface', self.patch_port_trunk_name,
                                  *patch_trunk_attrs))
+            txn.add(ovsdb.db_set('Port', self.patch_port_trunk_name,
+                                 ('vlan_mode', 'access'),
+                                 ('tag', tag)))
 
     def unplug(self, bridge):
         """Unplug the trunk from bridge.
@@ -156,7 +138,7 @@ class TrunkParentPort(object):
                        subport.
         """
         ovsdb = self.bridge.ovsdb
-        with self.ovsdb_transaction() as txn:
+        with ovsdb.transaction() as txn:
             txn.add(ovsdb.del_br(self.bridge.br_name))
             txn.add(ovsdb.del_port(self.patch_port_int_name,
                                    bridge.br_name))
@@ -189,12 +171,7 @@ class SubPort(TrunkParentPort):
         :param br_int: an integration bridge where peer endpoint of patch port
                        will be created.
         """
-        ovsdb = self.bridge.ovsdb
-        with self.ovsdb_transaction() as txn:
-            super(SubPort, self).plug(br_int)
-            txn.add(ovsdb.db_set(
-                "Port", self.patch_port_trunk_name,
-                ("tag", self.segmentation_id)))
+        super(SubPort, self).plug(br_int, tag=self.segmentation_id)
 
     def unplug(self, bridge):
         """Unplug the sub port from the bridge.
@@ -206,7 +183,7 @@ class SubPort(TrunkParentPort):
                        subport.
         """
         ovsdb = self.bridge.ovsdb
-        with self.ovsdb_transaction() as txn:
+        with ovsdb.transaction() as txn:
             txn.add(ovsdb.del_port(self.patch_port_trunk_name,
                                    self.bridge.br_name))
             txn.add(ovsdb.del_port(self.patch_port_int_name,
@@ -269,8 +246,8 @@ class TrunkManager(object):
             LOG.debug("Deleted bridge '%s' and patch peers '%s'.",
                       trunk_bridge.br_name, patch_peers)
         except RuntimeError as e:
-            LOG.error(_LE("Could not delete '%(peers)s' associated to "
-                          "trunk bridge %(name)s. Reason: %(reason)s."),
+            LOG.error("Could not delete '%(peers)s' associated to "
+                      "trunk bridge %(name)s. Reason: %(reason)s.",
                       {'peers': patch_peers,
                        'name': trunk_bridge.br_name,
                        'reason': e})
