@@ -253,6 +253,10 @@ class DietTestCase(base.BaseTestCase, metaclass=_CatchTimeoutMetaclass):
         # for more info.
         db_options.set_defaults(cfg.CONF, connection='sqlite://')
 
+        # NOTE(ykarel): Disable pool recycle as tables are dropped with sqlite
+        # memory db with connection close or reconnect
+        cfg.CONF.set_override('connection_recycle_time', -1, group='database')
+
         # Configure this first to ensure pm debugging support for setUp()
         debugger = os.environ.get('OS_POST_MORTEM_DEBUGGER')
         if debugger:
@@ -492,7 +496,11 @@ class BaseTestCase(DietTestCase):
                     root_helper_daemon=get_rootwrap_daemon_cmd())
 
     def _simulate_concurrent_requests_process_and_raise(self, calls, args):
+        self._simulate_concurrent_requests_process(calls, args,
+                                                   raise_on_exception=True)
 
+    def _simulate_concurrent_requests_process(self, calls, args,
+                                              raise_on_exception=False):
         class SimpleThread(threading.Thread):
             def __init__(self, q):
                 super(SimpleThread, self).__init__()
@@ -529,10 +537,16 @@ class BaseTestCase(DietTestCase):
             t.start()
         q.join()
 
+        threads_exceptions = []
         for t in threads:
             e = t.get_exception()
             if e:
-                raise e
+                if raise_on_exception:
+                    raise e
+                else:
+                    threads_exceptions.append(e)
+
+        return threads_exceptions
 
 
 class PluginFixture(fixtures.Fixture):
