@@ -94,13 +94,19 @@ class NeutronConfigFixture(ConfigFixture):
                 'report_interval': str(env_desc.agent_down_time // 2),
                 'log_agent_heartbeats': 'True',
             },
+            'quotas': {
+                'quota_driver': env_desc.quota_driver
+            },
+            'experimental': {
+                'linuxbridge': str(env_desc.allow_experimental_linuxbridge)
+            },
         })
 
         if use_local_apipaste:
             self.config['DEFAULT']['api_paste_config'] = (
                 self._generate_api_paste())
 
-        policy_file = self._generate_policy_json()
+        policy_file = self._generate_policy_yaml()
         if policy_file:
             self.config['oslo_policy'] = {'policy_file': policy_file}
 
@@ -131,6 +137,9 @@ class NeutronConfigFixture(ConfigFixture):
             self.config['DEFAULT']['network_scheduler_driver'] = (
                 env_desc.dhcp_scheduler_class)
 
+        self.config['DEFAULT']['enable_traditional_dhcp'] = str(
+            env_desc.enable_traditional_dhcp)
+
         net_helpers.set_local_port_range(CLIENT_CONN_PORT_START,
                                          CLIENT_CONN_PORT_END)
 
@@ -154,8 +163,8 @@ class NeutronConfigFixture(ConfigFixture):
     def _generate_api_paste(self):
         return c_helpers.find_sample_file('api-paste.ini')
 
-    def _generate_policy_json(self):
-        return c_helpers.find_sample_file('policy.json')
+    def _generate_policy_yaml(self):
+        return c_helpers.find_sample_file('policy.yaml')
 
     def get_host(self):
         return self.config['DEFAULT']['host']
@@ -171,13 +180,19 @@ class ML2ConfigFixture(ConfigFixture):
         if self.env_desc.l2_pop:
             mechanism_drivers += ',l2population'
 
+        net_vlan_ranges_extra = ''
+        if 'segments' in env_desc.service_plugins:
+            net_vlan_ranges_extra = (',' + PHYSICAL_NETWORK_NAME +
+                                     '_lb:1050:1059')
+
         self.config.update({
             'ml2': {
                 'tenant_network_types': tenant_network_types,
                 'mechanism_drivers': mechanism_drivers,
             },
             'ml2_type_vlan': {
-                'network_vlan_ranges': PHYSICAL_NETWORK_NAME + ':1000:1029',
+                'network_vlan_ranges': PHYSICAL_NETWORK_NAME + ':1000:1029' +
+                net_vlan_ranges_extra,
             },
             'ml2_type_gre': {
                 'tunnel_id_ranges': '1:30',
@@ -247,6 +262,18 @@ class OVSConfigFixture(ConfigFixture):
                     'local_output_log_base':
                         self._generate_temp_log_file(test_name)}
             })
+        if not env_desc.enable_traditional_dhcp:
+            self.config['agent']['extensions'] = 'dhcp'
+            self.config.update({
+                'dhcp': {
+                    'enable_ipv6': 'True',
+                    'dhcp_renewal_time': '0',
+                    'dhcp_rebinding_time': '0'}
+            })
+        if env_desc.local_ip_ext:
+            self.config['agent']['extensions'] = 'local_ip'
+            if host_desc.firewall_driver == 'openvswitch':
+                self.config['local_ip'] = {'static_nat': 'True'}
 
     def _setUp(self):
         self.config['ovs'].update({
@@ -337,6 +364,8 @@ class LinuxBridgeConfigFixture(ConfigFixture):
             env_desc, host_desc, temp_dir,
             base_filename="linuxbridge_agent.ini"
         )
+        self.service_plugins = env_desc.service_plugins
+
         self.config.update({
             'VXLAN': {
                 'enable_vxlan': str(self.env_desc.tunneling_enabled),
@@ -376,7 +405,10 @@ class LinuxBridgeConfigFixture(ConfigFixture):
             })
 
     def _generate_bridge_mappings(self, device_name):
-        return '%s:%s' % (PHYSICAL_NETWORK_NAME, device_name)
+        bridge_mappings_extra = ('_lb' if 'segments' in self.service_plugins
+                                 else '')
+        return '%s%s:%s' % (PHYSICAL_NETWORK_NAME, bridge_mappings_extra,
+                            device_name)
 
 
 class L3ConfigFixture(ConfigFixture):
