@@ -20,7 +20,6 @@ from neutron_lib.services.qos import constants as qos_consts
 from neutronclient.common import exceptions
 from oslo_utils import uuidutils
 
-from neutron.agent.linux import tc_lib
 from neutron.common import utils
 from neutron.tests.common.agents import l2_extensions
 from neutron.tests.fullstack import base
@@ -30,12 +29,6 @@ from neutron.tests.fullstack.resources import machine
 from neutron.tests.unit import testlib_api
 
 from neutron.agent.common import ovs_lib
-from neutron.conf.plugins.ml2.drivers import linuxbridge as \
-    linuxbridge_agent_config
-from neutron.plugins.ml2.drivers.linuxbridge.agent import \
-    linuxbridge_neutron_agent as linuxbridge_agent
-from neutron.services.qos.drivers.linuxbridge import driver as lb_drv
-from neutron.services.qos.drivers.openvswitch import driver as ovs_drv
 
 
 load_tests = testlib_api.module_load_tests
@@ -207,47 +200,6 @@ class _TestBwLimitQoS(BaseQoSRuleTestCase):
         for direction in list(all_directions):
             self._wait_for_bw_rule_applied(vm, None, None, direction)
 
-    def test_bw_limit_qos_policy_rule_lifecycle(self):
-        new_limit = BANDWIDTH_LIMIT + 100
-
-        # Create port with qos policy attached
-        vm, qos_policy = self._prepare_vm_with_qos_policy(
-            [functools.partial(
-                self._add_bw_limit_rule,
-                BANDWIDTH_LIMIT, BANDWIDTH_BURST, self.direction)])
-        bw_rule = qos_policy['rules'][0]
-
-        self._wait_for_bw_rule_applied(
-            vm, BANDWIDTH_LIMIT, BANDWIDTH_BURST, self.direction)
-        qos_policy_id = qos_policy['id']
-
-        self.client.delete_bandwidth_limit_rule(bw_rule['id'], qos_policy_id)
-        self._wait_for_bw_rule_removed(vm, self.direction)
-
-        # Create new rule with no given burst value, in such case ovs and lb
-        # agent should apply burst value as
-        # bandwidth_limit * qos_consts.DEFAULT_BURST_RATE
-        new_expected_burst = self._get_expected_burst_value(new_limit,
-                                                            self.direction)
-        new_rule = self.safe_client.create_bandwidth_limit_rule(
-            self.tenant_id, qos_policy_id, new_limit, direction=self.direction)
-        self._wait_for_bw_rule_applied(
-            vm, new_limit, new_expected_burst, self.direction)
-
-        # Update qos policy rule id
-        self.client.update_bandwidth_limit_rule(
-            new_rule['id'], qos_policy_id,
-            body={'bandwidth_limit_rule': {'max_kbps': BANDWIDTH_LIMIT,
-                                           'max_burst_kbps': BANDWIDTH_BURST}})
-        self._wait_for_bw_rule_applied(
-            vm, BANDWIDTH_LIMIT, BANDWIDTH_BURST, self.direction)
-
-        # Remove qos policy from port
-        self.client.update_port(
-            vm.neutron_port['id'],
-            body={'port': {'qos_policy_id': None}})
-        self._wait_for_bw_rule_removed(vm, self.direction)
-
     def test_bw_limit_direction_change(self):
         # Create port with qos policy attached, with rule self.direction
         vm, qos_policy = self._prepare_vm_with_qos_policy(
@@ -404,46 +356,6 @@ class TestBwLimitQoSOvs(_TestBwLimitQoS, base.BaseFullStackTestCase):
         self.assertIsNone(qos_queues)
 
 
-class TestBwLimitQoSLinuxbridge(_TestBwLimitQoS, base.BaseFullStackTestCase):
-    l2_agent_type = constants.AGENT_TYPE_LINUXBRIDGE
-    scenarios = [
-        ('egress', {'direction': constants.EGRESS_DIRECTION}),
-        ('ingress', {'direction': constants.INGRESS_DIRECTION}),
-    ]
-
-    @staticmethod
-    def _get_expected_burst_value(limit, direction):
-        # For egress bandwidth limit this value should be calculated as
-        # bandwidth_limit * qos_consts.DEFAULT_BURST_RATE
-        if direction == constants.EGRESS_DIRECTION:
-            return TestBwLimitQoSLinuxbridge._get_expected_egress_burst_value(
-                limit)
-        else:
-            return TestBwLimitQoSLinuxbridge._get_expected_ingress_burst_value(
-                limit)
-
-    @staticmethod
-    def _get_expected_ingress_burst_value(limit):
-        return int(
-            float(limit) /
-            float(linuxbridge_agent_config.DEFAULT_KERNEL_HZ_VALUE))
-
-    def _wait_for_bw_rule_applied(self, vm, limit, burst, direction):
-        port_name = linuxbridge_agent.LinuxBridgeManager.get_tap_device_name(
-            vm.neutron_port['id'])
-        tc = tc_lib.TcCommand(
-            port_name,
-            linuxbridge_agent_config.DEFAULT_KERNEL_HZ_VALUE,
-            namespace=vm.host.host_namespace
-        )
-        if direction == constants.EGRESS_DIRECTION:
-            utils.wait_until_true(
-                lambda: tc.get_filters_bw_limits() == (limit, burst))
-        elif direction == constants.INGRESS_DIRECTION:
-            utils.wait_until_true(
-                lambda: tc.get_tbf_bw_limits() == (limit, burst))
-
-
 class _TestDscpMarkingQoS(BaseQoSRuleTestCase):
 
     number_of_hosts = 2
@@ -540,15 +452,6 @@ class TestDscpMarkingQoSOvs(_TestDscpMarkingQoS, base.BaseFullStackTestCase):
             vm.bridge, vm.port.name, dscp_mark)
 
 
-class TestDscpMarkingQoSLinuxbridge(_TestDscpMarkingQoS,
-                                    base.BaseFullStackTestCase):
-    l2_agent_type = constants.AGENT_TYPE_LINUXBRIDGE
-
-    def _wait_for_dscp_marking_rule_applied(self, vm, dscp_mark):
-        l2_extensions.wait_until_dscp_marking_rule_applied_linuxbridge(
-            vm.host.host_namespace, vm.port.name, dscp_mark)
-
-
 class _TestPacketRateLimitQoS(BaseQoSRuleTestCase):
 
     number_of_hosts = 1
@@ -639,30 +542,6 @@ class TestPacketRateLimitQoSOvs(_TestPacketRateLimitQoS,
     ]
 
 
-class TestQoSWithL2Population(base.BaseFullStackTestCase):
-    scenarios = [
-        (constants.AGENT_TYPE_OVS,
-         {'mech_drivers': 'openvswitch',
-          'supported_rules': ovs_drv.SUPPORTED_RULES}),
-        (constants.AGENT_TYPE_LINUXBRIDGE,
-         {'mech_drivers': 'linuxbridge',
-          'supported_rules': lb_drv.SUPPORTED_RULES})
-    ]
-
-    def setUp(self):
-        host_desc = []  # No need to register agents for this test case
-        env_desc = environment.EnvironmentDescription(
-            qos=True, l2_pop=True, mech_drivers=self.mech_drivers)
-        env = environment.Environment(env_desc, host_desc)
-        super(TestQoSWithL2Population, self).setUp(env)
-
-    def test_supported_qos_rule_types(self):
-        res = self.client.list_qos_rule_types()
-        rule_types = {t['type'] for t in res['rule_types']}
-        expected_rules = set(self.supported_rules)
-        self.assertEqual(expected_rules, rule_types)
-
-
 class TestQoSPolicyIsDefault(base.BaseFullStackTestCase):
 
     NAME = 'fs_policy'
