@@ -13,6 +13,7 @@
 #    under the License.
 import datetime
 import functools
+import subprocess
 from unittest import mock
 
 import fixtures as og_fixtures
@@ -26,6 +27,7 @@ from oslo_utils import timeutils
 from oslo_utils import uuidutils
 from ovsdbapp.backend.ovs_idl import event
 from ovsdbapp.backend.ovs_idl import idlutils
+import tenacity
 
 from neutron.common.ovn import constants as ovn_const
 from neutron.common import utils as n_utils
@@ -144,6 +146,10 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
                 'port_id': port['id']}})
         return r1_f2
 
+    @tenacity.retry(
+        retry=tenacity.retry_if_exception_type(subprocess.TimeoutExpired),
+        wait=tenacity.wait_exponential(multiplier=0.02, max=1),
+        reraise=True)
     def _check_mac_binding_exists(self, macb_id):
         cmd = ['ovsdb-client', 'transact',
                self.mech_driver.sb_ovn.connection_string]
@@ -156,8 +162,7 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
         cmd += ['["OVN_Southbound", {"op": "select", "table": "MAC_Binding", '
                 '"where": [["_uuid", "==", ["uuid", "%s"]]]}]' % macb_id]
 
-        out, _ = processutils.execute(*cmd,
-                                      log_errors=False)
+        out, _ = processutils.execute(*cmd, log_errors=False, timeout=3)
         return str(macb_id) in out
 
     def test_floatingip_mac_bindings(self):
@@ -311,6 +316,17 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
         rows = cmd.execute(check_error=True)
         return rows[0] if rows else None
 
+    def _set_port_binding_virtual_parent(self, port_id, parent_port_id):
+        pb_port_parent = self.sb_api.db_find_rows(
+            'Port_Binding', ('logical_port', '=', parent_port_id)).execute(
+            check_error=True)[0]
+        pb_port_vip = self.sb_api.db_find_rows(
+            'Port_Binding', ('logical_port', '=', port_id)).execute(
+            check_error=True)[0]
+        self.sb_api.db_set(
+            'Port_Binding', pb_port_vip.uuid,
+            ('virtual_parent', pb_port_parent.uuid)).execute(check_error=True)
+
     def _check_port_binding_type(self, port_id, port_type):
         def is_port_binding_type(port_id, port_type):
             bp = self._find_port_binding(port_id)
@@ -340,6 +356,7 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
         port = self.create_port()
         self._check_port_binding_type(vip['id'], '')
 
+        # 1) Set the allowed address pairs.
         data = {'port': {'allowed_address_pairs': allowed_address_pairs}}
         req = self.new_update_request('ports', data, port['id'])
         req.get_response(self.api)
@@ -347,8 +364,15 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
         # and the corresponding "virtual-parents".
         self._check_port_binding_type(vip['id'], ovn_const.LSP_TYPE_VIRTUAL)
         self._check_port_virtual_parents(vip['id'], port['id'])
-        mock_update_vip_host.assert_not_called()
 
+        # 2) Unset the allowed address pairs.
+        # Assign the VIP again and delete the virtual port.
+        # Before unsetting the allowed address pairs, we first manually add
+        # the Port_Binding.virtual_parent of the virtual port. That happens
+        # when an ovn-controller detects traffic with the VIP and assign the
+        # port hosting the VIP as virtual parent.
+        self._set_port_binding_virtual_parent(vip['id'], port['id'])
+        mock_update_vip_host.reset_mock()
         data = {'port': {'allowed_address_pairs': []}}
         req = self.new_update_request('ports', data, port['id'])
         req.get_response(self.api)
@@ -356,8 +380,43 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
         self._check_port_virtual_parents(vip['id'], None)
         n_utils.wait_until_true(lambda: mock_update_vip_host.called,
                                 timeout=10)
+        # The virtual port is no longer considered as virtual. The
+        # "Port_Binding" register is deleted.
         mock_update_vip_host.assert_called_once_with(vip['id'], None)
 
+        # 3) Set again the allowed address pairs.
+        mock_update_vip_host.reset_mock()
+        data = {'port': {'allowed_address_pairs': allowed_address_pairs}}
+        req = self.new_update_request('ports', data, port['id'])
+        req.get_response(self.api)
+        # This test checks that the VIP "Port_Binding" register gets the type
+        # and the corresponding "virtual-parents".
+        self._check_port_binding_type(vip['id'], ovn_const.LSP_TYPE_VIRTUAL)
+        self._check_port_virtual_parents(vip['id'], port['id'])
+        mock_update_vip_host.reset_mock()
+        self._delete('ports', vip['id'])
+        n_utils.wait_until_true(lambda: mock_update_vip_host.called,
+                                timeout=10)
+        # The virtual port is deleted and so the associated "Port_Binding".
+        # With OVN v22.03.3 sometimes 2 delete events are received with the
+        # same arguments.
+        # TODO(lajoskatona): check when new OVN version is out
+        # if this behaviour is changed.
+        mock_update_vip_host.assert_called_with(vip['id'], None)
+
+    @mock.patch.object(mech_driver.OVNMechanismDriver,
+                       'update_virtual_port_host')
+    def test_non_virtual_port_no_host_update(self, mock_update_vip_host):
+        # The ``PortBindingUpdateVirtualPortsEvent`` delete event should affect
+        # only to virtual ports. This check is done for virtual ports in
+        # ``test_virtual_port_host_update``.
+        port = self.create_port()
+        self._delete('ports', port['id'])
+        # We actively wait for 5 seconds for the ``Port_Binding`` event to
+        # arrive and be processed, but the port host must not be updated.
+        self.assertRaises(n_utils.WaitTimeout, n_utils.wait_until_true,
+                          lambda: mock_update_vip_host.called, timeout=5)
+
 
 class TestNBDbMonitorOverTcp(TestNBDbMonitor):
     def get_ovsdb_server_protocol(self):
@@ -441,9 +500,8 @@ class TestAgentMonitor(base.TestOVNFunctionalBase):
             chassis_name, self.mech_driver.agent_chassis_table)
         self.mech_driver.sb_ovn.idl.notify_handler.watch_event(row_event)
         self.chassis_name = self.add_fake_chassis(
-            self.FAKE_CHASSIS_HOST,
-            external_ids={'ovn-cms-options': 'enable-chassis-as-gw'},
-            name=chassis_name)
+            self.FAKE_CHASSIS_HOST, name=chassis_name,
+            enable_chassis_as_gw=True)
         self.assertTrue(row_event.wait())
         n_utils.wait_until_true(
             lambda: len(list(neutron_agent.AgentCache())) == 1)
@@ -451,11 +509,11 @@ class TestAgentMonitor(base.TestOVNFunctionalBase):
     def test_agent_change_controller(self):
         self.assertEqual(neutron_agent.ControllerGatewayAgent,
                 type(neutron_agent.AgentCache()[self.chassis_name]))
-        self.sb_api.db_set('Chassis', self.chassis_name, ('external_ids',
+        self.sb_api.db_set('Chassis', self.chassis_name, ('other_config',
                 {'ovn-cms-options': ''})).execute(check_error=True)
         n_utils.wait_until_true(lambda:
                 neutron_agent.AgentCache()[self.chassis_name].
-                chassis.external_ids['ovn-cms-options'] == '')
+                chassis.other_config['ovn-cms-options'] == '')
         self.assertEqual(neutron_agent.ControllerAgent,
                 type(neutron_agent.AgentCache()[self.chassis_name]))
 
@@ -475,6 +533,12 @@ class TestAgentMonitor(base.TestOVNFunctionalBase):
         nb_cfg_timestamp = timestamp * 1000
         self.sb_api.db_set('Chassis_Private', self.chassis_name, (
             'nb_cfg_timestamp', nb_cfg_timestamp)).execute(check_error=True)
+        # Also increment nb_cfg by 1 to trigger ChassisAgentWriteEvent which
+        # is responsible to update AgentCache
+        old_nb_cfg = self.sb_api.db_get('Chassis_Private', self.chassis_name,
+            'nb_cfg').execute(check_error=True)
+        self.sb_api.db_set('Chassis_Private', self.chassis_name, (
+            'nb_cfg', old_nb_cfg + 1)).execute(check_error=True)
         try:
             n_utils.wait_until_true(check_agent_ts, timeout=5)
         except n_utils.WaitTimeout:
@@ -485,6 +549,66 @@ class TestAgentMonitor(base.TestOVNFunctionalBase):
             self.fail('Chassis timestamp: %s, agent updated_at: %s' %
                       (chassis_ts, str(agent.updated_at)))
 
+    def test_agent_restart(self):
+        def check_agent_up():
+            agent = neutron_agent.AgentCache()[self.chassis_name]
+            return agent.alive
+
+        def check_agent_down():
+            return not check_agent_up()
+
+        def check_nb_cfg_timestamp_is_not_null():
+            agent = neutron_agent.AgentCache()[self.chassis_name]
+            return agent.updated_at != 0
+
+        if not self.sb_api.is_table_present('Chassis_Private'):
+            self.skipTest('Ovn sb not support Chassis_Private')
+
+        # Set nb_cfg to some realistic value, so that the alive check can
+        # actually work
+        self.nb_api.db_set(
+            'NB_Global', '.', ('nb_cfg', 1337)).execute(check_error=True)
+        self.sb_api.db_set(
+            'Chassis_Private', self.chassis_name, ('nb_cfg', 1337)
+        ).execute(check_error=True)
+
+        chassis_uuid = self.sb_api.db_get(
+            'Chassis', self.chassis_name, 'uuid').execute(check_error=True)
+
+        self.assertTrue(check_agent_up())
+        n_utils.wait_until_true(check_nb_cfg_timestamp_is_not_null, timeout=5)
+
+        # Lets start by shutting down the ovn-controller
+        # (where it will remove the Chassis_Private table entry)
+        self.sb_api.db_destroy(
+            'Chassis_Private', self.chassis_name).execute(check_error=True)
+        try:
+            n_utils.wait_until_true(check_agent_down, timeout=5)
+        except n_utils.WaitTimeout:
+            self.fail('Agent did not go down after Chassis_Private removal')
+
+        # Now the ovn-controller starts up again and has not yet synced with
+        # the southbound database
+        self.sb_api.db_create(
+            'Chassis_Private', name=self.chassis_name,
+            external_ids={}, chassis=chassis_uuid,
+            nb_cfg_timestamp=0, nb_cfg=0
+        ).execute(check_error=True)
+        self.assertTrue(check_agent_down())
+
+        # Now the ovn-controller has synced with the southbound database
+        nb_cfg_timestamp = timeutils.utcnow_ts() * 1000
+        with self.sb_api.transaction() as txn:
+            txn.add(self.sb_api.db_set('Chassis_Private', self.chassis_name,
+                                       ('nb_cfg_timestamp', nb_cfg_timestamp)))
+            txn.add(self.sb_api.db_set('Chassis_Private', self.chassis_name,
+                                       ('nb_cfg', 1337)))
+        try:
+            n_utils.wait_until_true(check_agent_up, timeout=5)
+        except n_utils.WaitTimeout:
+            self.fail('Agent did not go up after sync is done')
+        self.assertTrue(check_nb_cfg_timestamp_is_not_null())
+
 
 class TestOvnIdlProbeInterval(base.TestOVNFunctionalBase):
     def setUp(self):
@@ -552,3 +676,45 @@ class TestOvnIdlConnections(base.TestOVNFunctionalBase):
                 **kwargs)
             self.assertEqual(set(idlutils.parse_connection(conns)),
                              set(idl._session.remotes))
+
+
+class TestPortBindingChassisEvent(base.TestOVNFunctionalBase,
+                                  test_l3.L3NatTestCaseMixin):
+
+    def setUp(self, **kwargs):
+        super().setUp(**kwargs)
+        self.chassis = self.add_fake_chassis('ovs-host1')
+        self.l3_plugin = directory.get_plugin(plugin_constants.L3)
+        kwargs = {'arg_list': (external_net.EXTERNAL,),
+                  external_net.EXTERNAL: True}
+        self.net = self._make_network(
+            self.fmt, 'ext_net', True, as_admin=True, **kwargs)
+        self._make_subnet(self.fmt, self.net, '20.0.10.1', '20.0.10.0/24')
+        port_res = self._create_port(self.fmt, self.net['network']['id'])
+        self.port = self.deserialize(self.fmt, port_res)['port']
+
+        self.ext_api = test_extensions.setup_extensions_middleware(
+            test_l3.L3TestExtensionManager())
+        self.pb_event_match = mock.patch.object(
+            self.sb_api.idl._portbinding_event, 'match_fn').start()
+
+    def _check_pb_type(self, _type):
+        def check_pb_type(_type):
+            if len(self.pb_event_match.call_args_list) < 1:
+                return False
+
+            pb_row = self.pb_event_match.call_args_list[0].args[1]
+            return _type == pb_row.type
+
+        n_utils.wait_until_true(lambda: check_pb_type(_type), timeout=5)
+
+    def test_pb_type_patch(self):
+        router = self._make_router(self.fmt, self._tenant_id)
+        self._add_external_gateway_to_router(router['router']['id'],
+                                             self.net['network']['id'])
+        self._check_pb_type('patch')
+
+    def test_pb_type_empty(self):
+        self.sb_api.lsp_bind(self.port['id'], self.chassis,
+                             may_exist=True).execute(check_error=True)
+        self._check_pb_type('')
