@@ -12,10 +12,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from itertools import chain
 from unittest import mock
 
+import netaddr
+
+from neutron_lib.db import api as db_api
 from oslo_utils import uuidutils
 
+from neutron.db import l3_attrs_db
 from neutron.objects.qos import binding as qos_binding
 from neutron.objects.qos import policy
 from neutron.objects import router
@@ -166,6 +171,24 @@ class RouterDbObjectTestCase(obj_test_base.BaseDbObjectTestCase,
         router_dict = router_obj.obj_to_primitive('1.0')
         self.assertNotIn('qos_policy_id', router_dict['versioned_object.data'])
 
+    def test_get_router_ids_without_router_std_attrs(self):
+        def create_r_attr_reg(idx):
+            with db_api.CONTEXT_WRITER.using(self.context):
+                router_db = {'id': self.objs[idx].id}
+                l3_attrs_db.ExtraAttributesMixin.add_extra_attr(self.context,
+                                                                router_db)
+
+        for idx in range(3):
+            self.objs[idx].create()
+        expected_router_ids = [r.id for r in self.objs]
+
+        for idx in range(3):
+            router_ids = router.Router.\
+                get_router_ids_without_router_std_attrs(self.context)
+            self.assertEqual(expected_router_ids, router_ids)
+            create_r_attr_reg(idx)
+            expected_router_ids = expected_router_ids[1:]
+
 
 class RouterPortIfaceObjectTestCase(obj_test_base.BaseObjectIfaceTestCase):
 
@@ -273,6 +296,51 @@ class FloatingIPDbObjectTestCase(obj_test_base.BaseDbObjectTestCase,
         self.assertNotIn('qos_network_policy_id',
                          obj_v1_1['versioned_object.data'])
 
+    def test_get_scoped_floating_ips(self):
+        def compare_results(router_ids, original_fips):
+            self.assertCountEqual(
+                original_fips,
+                [
+                    fip[0].id
+                    for fip in router.FloatingIP.get_scoped_floating_ips(
+                        self.context, router_ids)
+                ]
+            )
+
+        # Setup three routers, networks and external networks
+        routers = {}
+        for i in range(3):
+            router_id = self._create_test_router_id(name=f'router-{i}')
+            routers[router_id] = []
+            net_id = self._create_test_network_id()
+            fip_net_id = self._create_external_network_id()
+
+            # Create three subnets and three FIPs using the
+            # aforementioned networks and routers
+            for j in range(3):
+                self._create_test_subnet_id(net_id)
+                fip = router.FloatingIP(
+                    self.context,
+                    floating_ip_address=netaddr.IPAddress(f'10.{i}.{j}.3'),
+                    floating_network_id=fip_net_id,
+                    floating_port_id=self._create_test_port_id(
+                        network_id=fip_net_id),
+                    fixed_port_id=self._create_test_port_id(
+                        network_id=net_id),
+                    router_id=router_id,
+                )
+                fip.create()
+                routers[router_id].append(fip.id)
+
+        # For each router we created, fetch the fips and ensure the
+        # results match what we originally created
+        for router_id, original_fips in routers.items():
+            compare_results([router_id], original_fips)
+
+        # Now try to fetch all the fips for all the routers at once
+        original_fips = list(chain.from_iterable(routers.values()))
+        compare_results(routers.keys(), original_fips)
+
 
 class DvrFipGatewayPortAgentBindingTestCase(
         obj_test_base.BaseObjectIfaceTestCase):
