@@ -13,6 +13,7 @@
 import collections
 import itertools
 
+import netaddr
 from oslo_utils import uuidutils
 
 from neutron.objects import securitygroup
@@ -26,6 +27,7 @@ class SecurityGroupRBACDbObjectTestCase(test_rbac.TestRBACObjectMixin,
                                         testlib_api.SqlTestCase):
 
     _test_class = securitygroup.SecurityGroupRBAC
+    _parent_class = securitygroup.SecurityGroup
 
     def setUp(self):
         super(SecurityGroupRBACDbObjectTestCase, self).setUp()
@@ -72,10 +74,42 @@ class SecurityGroupDbObjTestCase(test_base.BaseDbObjectTestCase,
                 # generated; we picked the former here
                 rule['remote_group_id'] = None
 
+        sg_rule = self.get_random_db_fields(securitygroup.SecurityGroupRule)
+        self.model_map.update({
+            self._test_class.db_model: self.db_objs,
+            securitygroup.SecurityGroupRule.db_model: sg_rule})
+
     def _create_test_security_group(self):
         self.objs[0].create()
         return self.objs[0]
 
+    def _create_test_security_group_with_rule(self):
+        sg_obj = self._create_test_security_group()
+        rule_params = {
+            'project_id': sg_obj.project_id,
+            'security_group_id': sg_obj.id,
+            'remote_address_group_id': None,
+            'remote_ip_prefix': netaddr.IPNetwork('10.0.0.120/24')}
+        sg_rule = securitygroup.SecurityGroupRule(
+            self.context, **rule_params)
+        sg_obj.rules = [sg_rule]
+        return sg_obj
+
+    def test_object_version_degradation_1_3_to_1_2_no_remote_ag(self):
+        sg_obj = self._create_test_security_group_with_rule()
+        sg_obj_1_2 = sg_obj.obj_to_primitive('1.2')
+        for rule in sg_obj_1_2['versioned_object.data']['rules']:
+            self.assertEqual('1.0', rule['versioned_object.version'])
+            self.assertNotIn('remote_address_group_id',
+                             rule['versioned_object.data'])
+
+    def test_object_version_degradation_1_4_to_1_3_no_normalized_cidr(self):
+        sg_obj = self._create_test_security_group_with_rule()
+        sg_obj_1_3 = sg_obj.obj_to_primitive('1.3')
+        for rule in sg_obj_1_3['versioned_object.data']['rules']:
+            self.assertEqual('1.1', rule['versioned_object.version'])
+            self.assertNotIn('normalized_cidr', rule['versioned_object.data'])
+
     def test_object_version_degradation_1_2_to_1_1_no_stateful(self):
         sg_stateful_obj = self._create_test_security_group()
         sg_no_stateful_obj = sg_stateful_obj.obj_to_primitive('1.1')
@@ -176,6 +210,22 @@ class SecurityGroupDbObjTestCase(test_base.BaseDbObjectTestCase,
         self.assertEqual(len(sg_obj.rules), 0)
         self.assertIsNone(listed_objs[0].rules)
 
+    def test_get_sgs_stateful_flag(self):
+        for obj in self.objs:
+            obj.create()
+
+        sg_ids = tuple(sg.id for sg in self.objs)
+        sgs_stateful = securitygroup.SecurityGroup.get_sgs_stateful_flag(
+            self.context, sg_ids)
+        for sg_id, stateful in sgs_stateful.items():
+            for obj in (obj for obj in self.objs if obj.id == sg_id):
+                self.assertEqual(obj.stateful, stateful)
+
+        sg_ids = sg_ids + ('random_id_not_present', )
+        sgs_stateful = securitygroup.SecurityGroup.get_sgs_stateful_flag(
+            self.context, sg_ids)
+        self.assertEqual(len(self.objs), len(sgs_stateful))
+
 
 class DefaultSecurityGroupIfaceObjTestCase(test_base.BaseObjectIfaceTestCase):
 
@@ -213,9 +263,15 @@ class SecurityGroupRuleDbObjTestCase(test_base.BaseDbObjectTestCase,
                 'security_group_id':
                     lambda: self._create_test_security_group_id(),
                 'remote_group_id':
-                    lambda: self._create_test_security_group_id()
+                    lambda: self._create_test_security_group_id(),
+                'remote_address_group_id':
+                    lambda: self._create_test_address_group_id()
             })
 
+    def _create_test_security_group_rule(self):
+        self.objs[0].create()
+        return self.objs[0]
+
     def test_get_security_group_rule_ids(self):
         """Retrieve the SG rules associated to a project (see method desc.)
 
@@ -250,3 +306,15 @@ class SecurityGroupRuleDbObjTestCase(test_base.BaseDbObjectTestCase,
             rule_ids_ref = set(rules_per_project[projects[idx]])
             rule_ids_ref.update(set(rules_per_sg[sgs[idx]]))
             self.assertEqual(rule_ids_ref, set(rule_ids))
+
+    def test_object_version_degradation_1_1_to_1_0_no_remote_ag(self):
+        rule_remote_ag_obj = self._create_test_security_group_rule()
+        rule_no_remote_ag_obj = rule_remote_ag_obj.obj_to_primitive('1.0')
+        self.assertNotIn('remote_address_group_id',
+                         rule_no_remote_ag_obj['versioned_object.data'])
+
+    def test_object_version_degradation_1_2_to_1_1_no_normalized_cidr(self):
+        sg_rule_obj = self._create_test_security_group_rule()
+        sg_rule_10_obj = sg_rule_obj.obj_to_primitive('1.0')
+        self.assertNotIn('normalized_cidr',
+                         sg_rule_10_obj['versioned_object.data'])
